if Banker == nil then Banker = {} end

function initVars()
	PACKAGE = "com.github.Nols1000.banker"
	VERSION = 6

	Banker.enTypes = {
		[0] = ITEMTYPE_ADDITIVE,
		[1] = ITEMTYPE_ALCHEMY_BASE,
		[2] = ITEMTYPE_ARMOR,
		[3] = ITEMTYPE_ARMOR_BOOSTER,
		[4] = ITEMTYPE_ARMOR_TRAIT,
		[5] = ITEMTYPE_AVA_REPAIR,
		[6] = ITEMTYPE_BLACKSMITHING_BOOSTER,
		[7] = ITEMTYPE_BLACKSMITHING_MATERIAL,
		[8] = ITEMTYPE_BLACKSMITHING_RAW_MATERIAL,
		[9] = ITEMTYPE_CLOTHIER_BOOSTER,
		[10] = ITEMTYPE_CLOTHIER_MATERIAL,
		[11] = ITEMTYPE_CLOTHIER_RAW_MATERIAL,
		[12] = ITEMTYPE_COLLECTIBLE,
		[13] = ITEMTYPE_CONTAINER,
		[14] = ITEMTYPE_COSTUME,
		[15] = ITEMTYPE_DISGUISE,
		[16] = ITEMTYPE_DRINK,
		[17] = ITEMTYPE_ENCHANTING_RUNE_ASPECT,
		[18] = ITEMTYPE_ENCHANTING_RUNE_ESSENCE,
		[19] = ITEMTYPE_ENCHANTING_RUNE_POTENCY,
		[20] = ITEMTYPE_ENCHANTMENT_BOOSTER,
		[21] = ITEMTYPE_FLAVORING,
		[22] = ITEMTYPE_FOOD,
		[23] = ITEMTYPE_GLYPH_ARMOR,
		[24] = ITEMTYPE_GLYPH_JEWELRY,
		[25] = ITEMTYPE_GLYPH_WEAPON,
		[26] = ITEMTYPE_INGREDIENT,
		[27] = ITEMTYPE_LOCKPICK,
		[28] = ITEMTYPE_LURE,
		[29] = ITEMTYPE_NONE,
		[30] = ITEMTYPE_PLUG,
		[31] = ITEMTYPE_POTION,
		[32] = ITEMTYPE_RAW_MATERIAL,
		[33] = ITEMTYPE_REAGENT,
		[34] = ITEMTYPE_RECIPE,
		[35] = ITEMTYPE_SIEGE,
		[36] = ITEMTYPE_SOUL_GEM,
		[37] = ITEMTYPE_SPICE,
		[38] = ITEMTYPE_STYLE_MATERIAL,
		[39] = ITEMTYPE_TABARD,
		[40] = ITEMTYPE_TOOL,
		[41] = ITEMTYPE_TRASH,
		[42] = ITEMTYPE_TROPHY,
		[43] = ITEMTYPE_WEAPON,
		[44] = ITEMTYPE_WEAPON_BOOSTER,
		[45] = ITEMTYPE_WEAPON_TRAIT,
		[46] = ITEMTYPE_WOODWORKING_BOOSTER,
		[47] = ITEMTYPE_WOODWORKING_MATERIAL,
		[48] = ITEMTYPE_WOODWORKING_RAW_MATERIAL,
	}

	defaults = {}
	defaults.lang = 1
	defaults.iTypes = {}

	for i = 0, #Banker.enTypes-1, 1 do
		defaults.iTypes[Banker.enTypes[i]] = true
	end

	defaults.items   = true
	defaults.money   = true
	defaults.debug   = false
	defaults.msg     = true
	defaults.keymenu = true

	defaults.mStep  = 50
	defaults.mMin   = 500

	defaults.updated = false

	isBankOpen = false
end

function initKB()
	ZO_CreateStringId("SI_BINDING_NAME_SYNC_INVENTORY",     getStringByText(KB_SYNC_ITEMS))
	ZO_CreateStringId("SI_BINDING_NAME_AUTO_DEPOSIT_MONEY", getStringByText(KB_SAFE_MONEY))
end

function initUI()
	BankerKeybindStripDescriptor =
	{
		{ -- I think you can have more than one button in your group if you add more of these sub-groups
			alignment = KEYBIND_STRIP_ALIGN_CENTER,
			name = getStringByText(KB_SYNC_ITEMS),
			keybind = "SYNC_INVENTORY",
			callback = function() stackItems() end,
			visible = function() return isBankOpen end,
		},
		{ -- I think you can have more than one button in your group if you add more of these sub-groups
			alignment = KEYBIND_STRIP_ALIGN_CENTER,
			name = getStringByText(KB_SAFE_MONEY),
			keybind = "AUTO_DEPOSIT_MONEY",
			callback = function() safeMoney() end,
			visible = function() return isBankOpen end,
		},
	}

	KEYBIND_STRIP:AddKeybindButtonGroup(BankerKeybindStripDescriptor)
end

function onLoaded(event, name)
	if name ~= "Banker" then return end

	initVars()
	vars = ZO_SavedVars:New("Vars_Banker", 6, nil, defaults)
	vars.lang = GetCVar("language.2") or "en"

	if vars.lang ~= "en" or vars.lang ~= "de" or vars.lang ~= "fr" then
		mDebug(string.format("%s is not supported. Banker will use standard language(en).", vars.lang))
	end

	local LAM = LibStub:GetLibrary("LibAddonMenu-1.0")

	panelID = LAM:CreateControlPanel(PACKAGE, "|c5387FFBanker")
	LAM:AddHeader(panelID, PACKAGE .. ".header01", getStringByText(HEADER_SETTINGS))
	LAM:AddDescription(panelID, PACKAGE .. ".header01.desc", getStringByText(DESC_MAIN))
	LAM:AddCheckbox(panelID, PACKAGE .. ".header01.items",
						getStringByText(TITLE_EITEMS), "",
						function()
							return vars.items
						end,
						function (bool)
							vars.items =  bool
						end, false, "")
	LAM:AddCheckbox(panelID, PACKAGE .. ".header01.money",
						getStringByText(TITLE_EMONEY), "",
						function()
							return vars.money
						end,
						function (bool)
							vars.money =  bool
						end, false, "")
	LAM:AddHeader(panelID, PACKAGE .. ".header02", getStringByText(HEADER_ITEM_TYPES))
	for i = 0, #Banker.enTypes, 1 do
		LAM:AddCheckbox(panelID, string.format("%s.header02.type%s", PACKAGE, i),
						getStringByType(Banker.enTypes[i]), "",
						function()
							return vars.iTypes[Banker.enTypes[i]]
						end,
						function (bool)
							vars.iTypes[Banker.enTypes[i]] =  bool
						end, true, getStringByText(WARNING_ITEM_TYPES))
	end
	LAM:AddHeader(panelID, PACKAGE .. ".header03", getStringByText(HEADER_MONEY))
	LAM:AddSlider(panelID, PACKAGE .. ".header03.min", getStringByText(TITLE_MIN),
				  "Tooltip", 0, 5000, 50,
				  function()
					return vars.mMin
				  end,
				  function (arg0)
					vars.mMin = arg0
				  end, false, "")
	LAM:AddSlider(panelID, PACKAGE .. ".header03.step", getStringByText(TITLE_STEP),
				  "Tooltip", 0, 1000, 10,
				  function()
					return vars.mStep
				  end,
				  function (arg0)
					vars.mStep = arg0

					if arg0 < 1 then
						vars.mStep = 1
					end
				  end, false, "")
	LAM:AddHeader(panelID, PACKAGE .. ".header04", getStringByText(HEADER_DEVELOPER))
	LAM:AddCheckbox(panelID, PACKAGE .. ".header04.debug",
						getStringByText(TITLE_DEBUG), "",
						function()
							return vars.debug
						end,
						function (bool)
							vars.debug =  bool
						end, false, "")
	LAM:AddCheckbox(panelID, PACKAGE .. ".header04.msg",
						getStringByText(TITLE_MSG), "",
						function()
							return vars.msg
						end,
						function (bool)
							vars.msg =  bool
						end, false, "")
	LAM:AddCheckbox(panelID, PACKAGE .. ".header04.keymenu",
						getStringByText(TITLE_KEYMENU), "",
						function()
							return vars.keymenu
						end,
						function (bool)
							vars.keymenu =  bool
						end, false, "")

	initKB()
	initUI()
end

function onOpenBank()
	toggleSyncBinding()

	if vars.items then
		stackItems()
	end

	if vars.money then
		safeMoney()
	end
end

function onCloseBank()
	toggleSyncBinding()
end

function stackItems()
	local bankItems = getItems(BAG_BANK)
	local backpackItems = getItems(BAG_BACKPACK)

	for i = 0, #backpackItems-1, 1 do
		for k = 0, #bankItems-1, 1 do
			if backpackItems[i].name == bankItems[k].name then
				if vars.iTypes[backpackItems[i].itemType] then
					transferItem(BAG_BACKPACK, BAG_BANK, i, k)
				end
			end
		end
	end

	-- for i = 0, n(backpackItems)-1, 1 do
		-- for j = 0, n(enTypes)-1, 1 do
			-- if vars.iTypes[enTypes[j]] then
				-- if backpackItems[i] == enTypes[j] then
					-- for k = 0, n(bankItems)-1, 1 do

						-- local name = {}
						-- name[0] = GetItemName(BAG_BACKPACK, i):upper()
						-- name[1] = GetItemName(BAG_BANK, k):upper()

						-- if name[0] == name[1] then
							-- transferItem(BAG_BACKPACK, BAG_BANK, i, k)
						-- end
					-- end
				-- end
			-- end
		-- end
	-- end
end

function safeMoney()
	local bankMoney = GetBankedMoney()
	local playerMoney = GetCurrentMoney()

	if playerMoney > vars.mMin then
		local n = math.floor( (playerMoney - vars.mMin) / vars.mStep )

		if n > 0 then
			local tMoney = n * vars.mStep

			DepositMoneyIntoBank(tMoney)
			msg(string.format("%s |t16:16:EsoUI/Art/currency/currency_gold.dds|t were transferd to your bank.", tMoney))
		end
	end
end

function getItems(bagId)
	local items = {}
	local _, numBagSlots = GetBagInfo(bagId)

	for i = 0, numBagSlots, 1 do
		icon, _, _, _, _, _ = GetItemInfo(bagId, i)
		itemType = GetItemType(bagId, i)
		name = GetItemName(bagId, i)
		link = GetItemLink(bagId, i, Banker.linkStyle[0])
		items[i] = Banker:newItem(bagId, i, itemType, name, link, icon)
	end

	return items
end

function transferItem(fromBag, toBag, fromSlot, toSlot)

	local fStackSize = GetSlotStackSize(fromBag, fromSlot)
	local fName      = GetItemLink(fromBag, fromSlot)
	local tStackSize = GetSlotStackSize(toBag, toSlot)

	local stack      = (fStackSize + tStackSize) - 100

	if stack <= 0 and fStackSize ~= 0 then
		if stackItem(fromBag, fromSlot, toBag, toSlot, fStackSize, fName) then
			local texture = GetItemInfo(fromBag, fromSlot)
			msg(string.format("%s x [%s] was added to bank.", fStackSize, fName))
		else
			msg(string.format("[%s] wasn't added to bank.", fName))
		end
	elseif fStackSize ~= 0 then
		fStackSize = fStackSize - stack
		if fStackSize > 0 then
			if stackItem(fromBag, fromSlot, toBag, toSlot, fStackSize, fName) then
				local texture = GetItemInfo(fromBag, fromSlot)
				msg(string.format("%s x [%s] was added to bank.", fStackSize, fName))
			else
				msg(string.format("[%s] wasn't added to bank.", fName))
			end
		end
	end
end

function toggleSyncBinding()
	isBankOpen = not isBankOpen

	if vars.keymenu then
		if KEYBIND_STRIP:HasKeybindButtonGroup(BankerKeybindStripDescriptor) then
			KEYBIND_STRIP:UpdateKeybindButtonGroup(BankerKeybindStripDescriptor)
		else
			KEYBIND_STRIP:RemoveKeybindButtonGroup(BankerKeybindStripDescriptor)
			KEYBIND_STRIP:AddKeybindButtonGroup(BankerKeybindStripDescriptor)
		end
	end
end

function getStringByType(arg0)
	if vars.lang == "en" then
		return mType.en[arg0]
	elseif vars.lang == "de" then
		return mType.de[arg0]
	elseif vars.lang == "fr" then
		return mType.fr[arg0]
	end
end

function getStringByText(arg0)
	if vars.lang == "en" then
		return mText.en[arg0]
	elseif vars.lang == "de" then
		return mText.de[arg0]
	elseif vars.lang == "fr" then
		return mText.fr[arg0]
	end
end

function mDebug(arg0)
	if vars.debug then
		CHAT_SYSTEM:AddMessage("[BANKER] " .. arg0)
	end
end

function msg(arg0)
	if vars.msg then
		CHAT_SYSTEM:AddMessage(arg0)
	end
end

function Banker:newItem(iBagId, iSlotId, iType, iName, iLink, iIcon)
	local item ={
		bagId    = iBagId,
		slotId   = iSlotId,
		itemType = iType,
		name     = iName,
		link     = iLink,
		icon     = iIcon,
	}

	return item
end

EVENT_MANAGER:RegisterForEvent("BankerOnBankOpen", EVENT_OPEN_BANK, onOpenBank)
EVENT_MANAGER:RegisterForEvent("BankerOnBankClose", EVENT_CLOSE_BANK, onCloseBank)
EVENT_MANAGER:RegisterForEvent("BankerOnLoad", EVENT_ADD_ON_LOADED, onLoaded)

function stackItem(fromBag, fromSlot, toBag, toSlot, quantity, name)
    mDebug("stacking")
    local result = true
    -- just in case
    ClearCursor()
    -- must call secure protected (pickup the item via cursor)
    result = CallSecureProtected("PickupInventoryItem", fromBag, fromSlot, quantity)
    mDebug("called secure protected")
    if (result) then
        -- must call secure protected (drop the item on the cursor)
		mDebug("called secure protected2")
        result = CallSecureProtected("PlaceInInventory", toBag, toSlot)
    end
    -- clear the cursor to avoid issues
    ClearCursor()
    return result
end
